# Copyright 2017 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing Cloud Filestore locations."""

from __future__ import absolute_import
from __future__ import unicode_literals
from googlecloudsdk.api_lib.filestore import filestore_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.filestore.locations import flags
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


class List(base.ListCommand):
  """List all Cloud Filestore locations.

  ## EXAMPLE

  The following command lists a maximum of five Cloud Filestore locations
  sorted alphabetically by name in descending order:

    $ {command} --limit=5 --sort-by=~name
  """

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat(flags.LOCATIONS_LIST_FORMAT)

  def Run(self, args):
    """Make API calls to List Cloud Filestore locations.

    Args:
      args: gcloud command-line arguments.

    Returns:
      Generator that yields the Cloud Filestore locations.
    """
    project_ref = resources.REGISTRY.Parse(
        properties.VALUES.core.project.GetOrFail(),
        collection='file.projects')
    return filestore_client.FilestoreClient().ListLocations(
        project_ref, limit=args.limit)
